import ami
import boto3
from botocore.configloader import raw_config_parse


class Config:
    def __init__(self, client, args):
        self.client = client
        self._config = raw_config_parse("~/.spotr/config")['config']
        self._config.update({k: v for k, v in vars(args).iteritems() if v})

    def set_az(self, az):
        self._config['az'] = az

    @property
    def ami_tag(self):
        return self._config.get('ami_tag', 'spotr')

    @property
    def instance_tag(self):
        return self._config.get('instance_tag', 'spotr')

    @property
    def type(self):
        return self._config['type']

    @property
    def max_bid(self):
        return self._config.get('max_bid', None)

    @property
    def ami(self):
        if 'ami' not in self._config:
            self._config['ami'] = ami.get_by_tag(self.client, self.ami_tag)
        return self._config['ami']

    @property
    def key_name(self):
        return self._config['key_name']

    @property
    def az(self):
        return self._config['az']



import pricing
import spot_instance
import client as boto_client
from config import Config


def launch(args):
    client = boto_client.build(args)
    conf = Config(client, args)

    az = pricing.get_az(client, conf)
    _log_launching(az)
    conf.set_az(az.zone_name)

    instance = spot_instance.request(client, conf)
    _log_instance_creation(instance, conf)


def _log_launching(az):
    print(">> Launching instance in:")
    print(str(az))


def _log_instance_creation(instance, conf):
    print(">> Instance launched, connect with:")
    ip = str(instance.ip_address)
    key_path = "~/.ssh/" + conf.key_name + ".pem"
    print("ssh -i " + key_path + " ubuntu@" + ip)



import argparse

import launch
import destroy

parser = argparse.ArgumentParser()
parser.add_argument('--version', action='version', version='0.0.3')
subparsers = parser.add_subparsers()

launch_parser = subparsers.add_parser('launch')
launch_parser.add_argument(
    '--type',
    help='the type of the instance to launch, eg p2.xlarge2')
launch_parser.add_argument(
    '--region',
    help='the region to launch the instance in')
launch_parser.add_argument(
    '--aws-access-key-id',
    help='the access key id to use')
launch_parser.add_argument(
    '--aws-secret-access-key',
    help='the secret access key to use')
launch_parser.add_argument(
    '--max-bid',
    help='the maximum bid price')
launch_parser.add_argument(
    '--ami',
    help='id of ami to use')
launch_parser.add_argument(
    '--ami-tag',
    help='the tag to search for the AMI by')
launch_parser.add_argument(
    '--key-name',
    help='name of the aws key pair to use')
launch_parser.set_defaults(func=launch.launch)

destroy_parser = subparsers.add_parser('destroy')
destroy_parser.add_argument(
    '--region',
    help='the region to launch the instance in')
destroy_parser.add_argument(
    '--aws-access-key-id',
    help='the access key id to use')
destroy_parser.add_argument(
    '--aws-secret-access-key',
    help='the secret access key to use')
destroy_parser.set_defaults(func=destroy.destroy)


def main():
    args = parser.parse_args()
    args.func(args)


if __name__ == '__main__':
    main()



import launch
import snapshot
import instance as ec2_instance
import client as boto_client
from config import Config


def destroy(args):
    client = boto_client.build(args)
    conf = Config(client, args)
    instance = ec2_instance.find_latest(client, conf)
    _log_snapshot(instance)
    snap = snapshot.create_and_wait(client, instance.volume_id)
    _log_destroying()
    ec2_instance.destroy(client, instance.id)


def _log_snapshot(instance):
    print(">> Creating snapshot for instance:")
    print(str(instance.ip_address))


def _log_destroying():
    print(">> Destroying instance.")



#!/usr/bin/env python
import codecs
import os.path
import re
import sys
from setuptools import find_packages, setup
from spotr import __version__

here = os.path.abspath(os.path.dirname(__file__))

def read(*parts):
    return codecs.open(os.path.join(here, *parts), 'r').read()

def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


requires = ['boto3==1.4.7']

setup(
    name = 'spotr',
    version = __version__,
    description = '',
    long_description=open('README.rst').read(),
    url = 'https://github.com/samuelreh/spotr',
    author = 'Samuel Reh',
    author_email = 'samuelreh@gmail.com',
    license = 'UNLICENSE',
    classifiers = [
        'Intended Audience :: Developers',
        'Topic :: Utilities',
        'License :: Public Domain',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    keywords = 'cli',
    packages = find_packages(exclude=['docs', 'tests*']),
    install_requires = ['docopt'],
    extras_require = {
        'test': ['coverage', 'pytest', 'pytest-cov'],
    },
    entry_points = {
        'console_scripts': [
            'spotr=spotr.spotr:main',
        ],
    },
)



import time
import random

import instance


def request(client, config):
    request_id = _perform_request(client, config)
    _wait_until_completed(client, request_id)
    instance_id = _get_status(client, request_id)
    instance.tag(client, instance_id, config)
    return instance.get_by_instance_id(client, instance_id)


def _perform_request(client, config):
    random_id = str(random.random() * 1000)
    response = client.request_spot_instances(
        SpotPrice=config.max_bid,
        ClientToken=random_id,
        InstanceCount=1,
        Type='one-time',
        LaunchSpecification={
            'ImageId': config.ami,
            'KeyName': config.key_name,
            'InstanceType': config.type,
            'Placement': {
                'AvailabilityZone': config.az,
            }
        }
    )
    return response.get('SpotInstanceRequests')[0].get('SpotInstanceRequestId')


def _wait_until_completed(client, request_id):
    waiter = client.get_waiter('spot_instance_request_fulfilled')
    return waiter.wait(SpotInstanceRequestIds=[request_id])


def _get_status(client, request_id):
    response = client.describe_spot_instance_requests(
        SpotInstanceRequestIds=[request_id],
    )
    return response.get('SpotInstanceRequests')[0].get('InstanceId')
